#include "broken.h"

#define LTE_MON_IDLE_TIMEOUT       (3 * 1000)      // in milliseconds

#define FAIL_FAST 1

#if FAIL_FAST
static const u32_t powerup_wait = 1;
static const u32_t powerdown_wait = 1;
static const u32_t on_time = 500;
static const u32_t off_time = 500;
#else
static const u32_t powerup_wait = 180;
static const u32_t powerdown_wait = 10;
static const u32_t on_time = K_SECONDS(60);
static const u32_t off_time = K_SECONDS(60);
#endif

const char *now( void )
{
    static char buff[64];

    u32_t ms = k_uptime_get_32();
    u32_t hours = ms / (60 * 60 * 1000);
    u32_t minutes = (ms / (60 * 1000)) % 60;
    u32_t seconds = (ms / 1000) % 60;
    ms = ms % 1000;

    sprintf( buff, "%02d:%02d:%02d.%03d", hours, minutes, seconds, ms );
    return( buff );
}

static
void modem_poweron( bool on )
{
    Modem modem = MODEM_INIT;

    if( modem_open( &modem ) )
        return;

    modem_cmd( &modem, "AT+CMEE=1", 1000 );         // report errors as "+CME ERROR: xxx"
    if( on )
    {
        modem_cmd( &modem, "AT+CEMODE=2", 1000 );       // CS/PS mode 2
#if FAIL_FAST
        modem_cmd( &modem, "AT+CFUN=4", 10000 );
#else
        modem_cmd( &modem, "AT+CFUN=1", 10000 );        // takes a few seconds
#endif
    }
    else
    {
        modem_cmd( &modem, "AT+CFUN=4", 10000 );        // takes a few seconds
    }

    modem_close( &modem );
}

void modem_query( void )
{
    Modem modem = MODEM_INIT;

    if( modem_open( &modem ) )
        return;

    modem_cmd( &modem, "AT+COPS?", 1000 );
    modem_cmd( &modem, "AT+CEREG?", 1000 );
    modem_cmd( &modem, "AT+CESQ", 1000 );
    modem_cmd( &modem, "AT+CIMI", 1000 );       // IMSI
    modem_cmd( &modem, "AT%XMONITOR", 1000 );         // TODO: includes COPS and CEREG data so use this instead
    modem_cmd( &modem, "AT+CGDCONT?", 1000 );         // get our IP address (don't think we care)
    modem_cmd( &modem, "AT+CGPADDR=0", 1000 );        // read IP address, context 0
    modem_cmd( &modem, "AT+CGCONTRDP=0", 1000 );      // read PDP params (eg DNS), context 0

    modem_close( &modem );

    return;
}

static bool is_connected = false;

void got_connect_event( bool connected )
{
    DBG( "%s\n", connected ? "CONNECTED" : "DISCONNECTED" );
    is_connected = connected;
    if( is_connected )
        modem_query();
}

// no error checking
static
void waitfor( u32_t timeout, bool on )
{
    DBG( "wait up to %d seconds\n", timeout );

    u32_t save = timeout;
    do
    {
        //DBG( "is_connected %d, on %d, timeout %d\n", is_connected, on, timeout );
        if( is_connected == on )
        {
            DBG( "elapsed %d seconds\n", save - timeout );
            return;
        }
        k_sleep( 1000 );
    }
    while( --timeout > 0 );
}

// the monitor thread watches for notifications from the modem so we don't have to poll
static
void monitor( void *unused1, void *unused2, void *unused3 )
{
    Modem modem = MODEM_INIT;

    while( 1 )
    {
        if( modem.sock == -1 )
        {
            // need to restart the socket to recover from error
            modem_open( &modem );

            // enable error reporting like "+CME ERROR: xxx" vice "ERROR"
            modem_cmd( &modem, "AT+CMEE=1", 1000 );

            modem_cmd( &modem, "AT+CFUN?", 1000 );      // current mode
            modem_cmd( &modem, "AT%XSYSTEMMODE?", 1000 );   // current mode

            // get some basic, static information from the modem
            modem_cmd( &modem, "AT+CGMI", 1000 );       // manufacturer, aka "Nordic ..."
            modem_cmd( &modem, "AT+CGMM", 1000 );       // model number, eg. nRF9160-SICA
            modem_cmd( &modem, "AT+CGMR", 1000 );       // version, eg mfw_nrf9160_1.0.0
            modem_cmd( &modem, "AT+CGSN", 1000 );       // serial number
            modem_cmd( &modem, "AT+CIMI", 1000 );       // IMSI (fails until modem has been turned on)

            // subscribe to statuses
            modem_cmd( &modem, "AT+CEREG=5", 1000 );      // registration status
            modem_cmd( &modem, "AT%CESQ=1", 1000 );       // signal quality
            modem_cmd( &modem, "AT+CGEREP=1", 1000 );     // packet domain event reporting
            modem_cmd( &modem, "AT+CNEC=24", 1000 );      // enable unsolicited network error codes
            modem_cmd( &modem, "AT+CIND=1,1,1", 1000 );   // report service indicators (copied from link monitor app)
            modem_cmd( &modem, "AT%XSIM=1", 1000 );       // subscribe UICC notifications
        }

        // do a long timeout on the readline. the modem sometimes stops responding
        if( modem_readline( &modem, LTE_MON_IDLE_TIMEOUT ) == NULL )
        {
            // badness. restart the connection
            DBG( "re-opening the modem, fd %d\n", modem.sock );
            modem_close( &modem );
            continue;
        }
    }
}

static
void abuse( void *unused1, void *unused2, void *unused3 )
{

    // alternate turning power on and off
    while( 1 )
    {
        DBG( "poweron modem\n" );
        modem_poweron( true );
        //waitfor( powerup_wait, true );

        //DBG( "leave modem on for %ds\n", on_time / 1000 );
        //k_sleep( on_time );

        DBG( "poweroff modem\n" );
        modem_poweron( false );
        //waitfor( powerdown_wait, false );

        //DBG( "sleep for %ds\n", off_time / 1000 );
        //k_sleep( off_time );
    }
}

K_THREAD_DEFINE( monitor_tid, 4096, monitor, NULL, NULL, NULL, 5, 0, 0 );
K_THREAD_DEFINE( abuse1_tid, 4096, abuse, NULL, NULL, NULL, 5, 0, 0 );
K_THREAD_DEFINE( abuse2_tid, 4096, abuse, NULL, NULL, NULL, 5, 0, 0 );
K_THREAD_DEFINE( abuse3_tid, 4096, abuse, NULL, NULL, NULL, 5, 0, 0 );
K_THREAD_DEFINE( abuse4_tid, 4096, abuse, NULL, NULL, NULL, 5, 0, 0 );

void main( void )
{
    // wait for monitor thread to start
    k_sleep( 1000 );
}
